/* Build Webpack4 config
 * @Author:             Kvkens(yueming@yonyou.com)
 * @Date:               2019-01-22 14:57:43
 * @Last Modified by:   Kvkens
 * @Last Modified time: 2019-06-21 16:39:15
 */
const { merge } = require("webpack-merge");
const argv = require("minimist")(process.argv.slice(2));
const commands = argv;
const util = require("../src/util");
const base = require("./base.config");
const cfg = util.getUcfConfig(commands._);
const htmlConfList = util.htmlWebpackPluginHook(cfg, "prod");
const pluginConfig = cfg.pluginConfig;
const entries = {};
htmlConfList.forEach((htmlItem) => {
    let { entryKey, entryChunk } = htmlItem;
    entries[entryKey] = entryChunk;
});

let splitChunks = {
    cacheGroups: {
        default: {
            minChunks: 2,
            priority: -20,
            reuseExistingChunk: true,
        },
        //打包重复出现的代码
        vendor: {
            chunks: "initial",
            minChunks: 2,
            maxInitialRequests: 5, // The default limit is too small to showcase the effect
            minSize: 0, // This is example is too small to create commons chunks
            name: "vendor",
        },
        //打包第三方类库
        commons: {
            name: "vendor",
            chunks: "initial",
            minChunks: Infinity,
        },
    },
};

//默认的配置用于merge操作
const config = {
    entry: entries,
    output: base.output,
    mode: "production",
    devtool: util.defaultExtendFunc(cfg.devtool, false),
    optimization: {
        minimize: commands.mode === "dev" ? false : true,
        minimizer: [
            util.pluginConfigHook(pluginConfig, "CssMinimizerPlugin"),
            util.pluginConfigHook(pluginConfig, "TerserPlugin"),
        ],
        splitChunks,
    },
    plugins: util.defaultExtendFunc(
        util.pluginConfigHook(pluginConfig,'plugins'),
        [
            util.pluginConfigHook(pluginConfig, "CopyWebpackPlugin"),
            util.pluginConfigHook(pluginConfig, "CleanWebpackPlugin"),
            ...util.pluginConfigHook(pluginConfig, "HtmlWebPackPlugin", htmlConfList),
        ]
    ),
};

module.exports = util.configHandler(cfg.webpackCfg, merge(base, config));
